function c_subpix = cornerSubPix( I, c_px, win_size )
%CORNERSUBPIX this function find the sunpixel position of a corner
%   this is a minimal implementation, for details refer to:
%   http://docs.opencv.org/master/dd/d1a/group__imgproc__feature.html#ga354e0d7c86d0d9da75de9b9701a9a87e&gsc.tab=0
%   and the opencv source code.
%   Matlab versions can be find in camera calibration toolboxes like
%     http://www.vision.caltech.edu/bouguetj/
%     https://sites.google.com/site/scarabotix/ocamcalib-toolbox

% check size
input_size = size(c_px);
if input_size(1) ~= 2 || input_size(2) ~= 1
    disp('Input corner must be a 2x1 vector (col, row)');
    return
end

% pad patch size to odd to
if mod(win_size, 2) == 0
    win_size = win_size + 1;
end
half_win = floor(win_size / 2);

% check the patch will be in border
img_size = size(I);
if c_px(1) <= half_win+1 || c_px(2) <= half_win+1 || ...
        c_px(1) >= img_size(2)-half_win || c_px(2) >= img_size(1)-half_win
    disp('Corner position is too close to the border, return the original value');
    c_subpix = c_px;
    return
end

% [col; row] => [row; col]
c_px = flipud(c_px); 

% set iteration parameters
max_iter = 10;
stop_eps = 0.005;

% subpixel refinement
c_cur = c_px;
c_update = [0; 0];
dx = [-1; 0; 1];
dy = [-1, 0, 1];
for i = 1:max_iter
    if i>1 && norm(c_update) < stop_eps
        break;
    end
    
    % get the window: 1 pixel farther at the border
    subpix_area = I(c_px(1)-half_win-1:c_px(1)+half_win+1, ...
        c_px(2)-half_win-1:c_px(2)+half_win+1);
    index_x = zeros(win_size);
    index_y = zeros(win_size);
    for j = 1:win_size
        index_x(j, :) = ones(1, win_size)*(c_px(1)-half_win+j-1);
        index_y(:, j) = ones(win_size, 1)*(c_px(2)-half_win+j-1);
    end
    
    % calculate the gradient
    Ix = imfilter(subpix_area, dx, 'replicate');
    Iy = imfilter(subpix_area, dy, 'replicate');
    Ix = Ix(2:win_size+1, 2:win_size+1);
    Iy = Iy(2:win_size+1, 2:win_size+1);
    Ixx = Ix.*Ix;
    Iyy = Iy.*Iy;
    Ixy = Ix.*Iy;
    
    % form the normal equation for the least square problem
    sum_Ixx = sum(Ixx(:));
    sum_Iyy = sum(Iyy(:));
    sum_Ixy = sum(Ixy(:));
    A = [sum_Ixx, sum_Ixy; sum_Ixy, sum_Iyy];
    IxxPx = Ixx.*index_x;
    IyyPy = Iyy.*index_y;
    IxyPy = Ixy.*index_y;
    IxyPx = Ixy.*index_x;
    
    b = [sum(IxxPx(:)) + sum(IxyPy(:)); sum(IxyPx(:)) + sum(IyyPy(:))];
    
    % solve for c_opt
    c_opt = A\b;
    
    % update and calculate the difference
    c_update = c_opt - c_cur;
    c_cur = c_opt;
end

c_subpix = flipud(c_cur); % [row; col] => [col; row]

end

