function [ corners ] = harrisCorners( I, patch_size, k, R_threshold, display_intermediate_results )
%HARRISCORNERS Summary of this function goes here
%   Detailed explanation goes here


%% TASK 1: Compute image gradients in x and y direction

% Create filter kernels
filter_kernel_dx = fspecial('sobel');
filter_kernel_dy = filter_kernel_dx';  %% DELETE

% Filter the input image
I_x = imfilter(I, filter_kernel_dx, 'replicate');
I_y = imfilter(I, filter_kernel_dy, 'replicate');

% Display results
if display_intermediate_results
    figure('Name', 'Intermediate Results', 'NumberTitle', 'off') 
    subplot(3,3,1)
    imshow(I_x,[])
    title('I_x')
    subplot(3,3,2)
    imshow(I_y,[])
    title('I_y')
end
    
%% TASK 2: Compute components of 2nd moment matrix for every pixel

I_xx = I_x.*I_x;
I_yy = I_y.*I_y; %% DELETE
I_xy = I_x.*I_y; %% DELETE

% Apply box filter (try with an average filter and compare results)
filter_kernel_gauss = fspecial('gaussian', [patch_size, patch_size]);
I_xx = imfilter(I_xx, filter_kernel_gauss, 'replicate');
I_yy = imfilter(I_yy, filter_kernel_gauss, 'replicate'); %% DELETE
I_xy = imfilter(I_xy, filter_kernel_gauss, 'replicate'); %% DELETE

% Display results
if display_intermediate_results
    subplot(3,3,4)
    imshow(I_xx,[])
    title('\Sigma I_x^2')
    subplot(3,3,5)
    imshow(I_yy,[])
    title('\Sigma I_y^2')
    subplot(3,3,6)
    imshow(I_xy,[])
    title('\Sigma I_xI_y')
end

%% TASK 3: Compute Harris score

[height, width] = size(I);
R = (I_xx.*I_yy - I_xy.^2) - k*(I_xx + I_yy).^2; %% DELETE, implement this in one line of code

% If you implement this line with two for loops, it would work like this:
%
% R = zeros(height, width);   % Harris corner score
% L = zeros(4, height*width); % Eigenvalues
% for y = 1:height
%    for x = 1:width
%       M = [I_xx(y,x), I_xy(y,x);
%            I_xy(y,x), I_yy(y,x)];
%        
%       % Harris cornerness function
%       R(y,x) = det(M) - k*trace(M)^2;
%       
%       % just for visualization, compute the eigenvalues of M
%       [V,D] = eig(M);
%       L(1:2,y*x) = diag(V*D*V); % we could just use eig(M) but then the eigenvalues would be sorted
%    end
% end

% Display results
if display_intermediate_results
    subplot(3,3,7)
    imshow(R, [])
    title('R')    
end

%% Threshold results
R_thresholded = R > R_threshold;

if display_intermediate_results
    subplot(3,3,8)
    imshow(R_thresholded,[])
    title('R thresholded')    
end

%% Non-Maxima suppression

nonmax_radius = 5;                                   
mx = ordfilt2(R,nonmax_radius^2,ones(nonmax_radius)); % Grey-scale dilate.
cm = (R==mx) & R_thresholded;       

[r,c] = find(cm);                  % Find row,col coords.
corners = [c';r'];

if display_intermediate_results
    subplot(3,3,9)
    imshow(I, [])
    hold on
    plot(c, r,'y.')
    title('nonmaxima suppression')  
end

end

