% Example of Harris corner detection
%
% Christian Forster

% Harris corner detector according to:
% C. Harris and M. Stephens, "A combined corner and edge detector," 
% in Proceedings of The Fourth Alvey Vision Conference, pp. 147-151, 1988. 

% You can evaluate one code block by pressing Ctrl+ENTER
% Or, you can evaluate one or more lines by selecting them and pressing the
% F9 key (without needing to copy them to the prompt in the Command Window)

clear all % clear all variables in the workspace
close all % close all figures
clc       % clear the command window

% Read an image from file
filename = 'tiger.jpg';
I = imread(filename);

% Convert image from RGB to grayscale
if size(size(I), 2) == 3
  I = double(rgb2gray(I) )/255;
else
  I = double(I);
end

% Parameters
% TODO: change patch_size and k (sensitive factor) to observe
%       how the result changes with these parameters
patch_size = 5;
k = 0.04;

R_threshold = 0.05;
display_intermediate_results = true;

% Display image
figure('Name', 'Original Image', 'NumberTitle', 'off')
imshow(I, [])
axis on

%% Extract harris corners

corners = harrisCorners(I, patch_size, k, R_threshold, display_intermediate_results);

%% Display corners
figure('Name', 'Corners Detected', 'NumberTitle', 'off')
imshow(I, [])
hold on
plot(corners(1,:), corners(2,:) ,'ys')

%% Subpixel refine

% TODO: uncomment this section and compare with the
%       result without subpixel refine

% corner_number = length(corners);
% corners_subpix = zeros(size(corners));
% 
% % for each corner, find the subpixel value in its neighborhood
% for i=1:corner_number
%     corners_subpix(:, i) = cornerSubPix(I, corners(:, i), 5);
% end
% 
% % remove duplicates
% corners_final = unique(corners_subpix', 'rows')';
% 
% % display
% figure('Name', 'Subpixel Corners', 'NumberTitle', 'off')
% imshow(I, [])
% hold on
% plot(corners_subpix(1,:), corners_subpix(2,:) ,'ys')